package org.eclipse.palamedes.gdl.core.ast;

import org.eclipse.palamedes.kif.core.ast.KIFImplication;
import org.eclipse.palamedes.kif.core.ast.KIFRelSent;
import org.eclipse.palamedes.kif.core.ast.KIFSent;

/**
 * A rule is an implication of the form
 * <code> h <= b_1 &and; ... &and; b_n </code><br>
 * <ul>
 * <li>h is an atomic sentence</li>
 * <li>b_i is a literal or a disjunction of literals</li>
 * </ul>
 *
 * @author Ingo Keller
 *
 */
public abstract class Rule extends KIFImplication {

    /**
     * The constructor checks whether the node is a rule. The criteria is: - the
     * consequence has to be a KIFRelSent object
     *
     * @param node
     *            Implication with a rule inside.
     */
    Rule(KIFImplication node) {
        super(node.getAST());
        copyMembers(node);

        // implication has to have a consequence
        if (node.getConsequence() == null)
            throw new IllegalArgumentException();

        // consequence has to be a KIFRelSent
        KIFSent head = node.getConsequence();
        if ( !(head instanceof KIFRelSent) )
            throw new IllegalArgumentException();

    }

    @Override
    public KIFRelSent getConsequence() {
        return (KIFRelSent) super.getConsequence();
    }

    @Override
    public void setConsequence(KIFSent consequence) {
        if (!(consequence instanceof KIFRelSent))
            throw new IllegalArgumentException(GDLAST.error1);
        super.setConsequence(consequence);
    }

    public String getHeadLvl1Const() {
        return getConsequence().getName();
    }

    public String getHeadLvl2Const() {
        if (getConsequence().getTerms().isEmpty())
            return "";

        return getConsequence().getTerms().get(0).getName();
    }

}
