package org.eclipse.palamedes.gdl.core.knowledgebase.analyser;

import java.util.Stack;

import org.eclipse.palamedes.kif.core.ast.ASTNode;
import org.eclipse.palamedes.kif.core.ast.KIFConjunction;
import org.eclipse.palamedes.kif.core.ast.KIFDisjunction;
import org.eclipse.palamedes.kif.core.ast.KIFImplication;
import org.eclipse.palamedes.kif.core.ast.KIFNegation;
import org.eclipse.palamedes.kif.core.ast.KIFRelSent;
import org.eclipse.palamedes.kif.core.ast.KIFSent;
import org.eclipse.palamedes.kif.core.knowledgebase.IKBAnalyser;
import org.eclipse.palamedes.kif.core.knowledgebase.KBEntry;
import org.eclipse.palamedes.kif.core.knowledgebase.KnowledgeBase;


public class AEnvironment implements IKBAnalyser {

    private KnowledgeBase kb;

    public void analyse(KnowledgeBase knowledgeBase) {
        kb = knowledgeBase;
        resolveEnvironmentLookup();
    }

    /**
     * This method finds all predicates that perform a environment lookup.
     */
    private void resolveEnvironmentLookup() {

        Stack<KBEntry> queue = new Stack<KBEntry>();

        for (KBEntry description : kb.getEntries())
            queue.push(description);

        // iterate over all descriptions as long as something changes.
        while ( !queue.isEmpty() ) {

            // get top stack element
            KBEntry info = queue.pop();

            // if no implication heads nothing can be done here
            if ( info.getImplicationHead().size() == 0 )
                continue;

            // if all premises of all implication heads are based on:
            // a) itself
            // b) true statements
            // c) environment lookup statements
            // then it is a environment lookup
            boolean isEnvironmentLookup = true;
            for (KIFImplication rule : info.getImplicationHead())
                if ( !isEnvironmentLookup(rule) ) {
                    isEnvironmentLookup = false;
                    break;
                }

            // resolved new lookup predicate
            if (isEnvironmentLookup) {
                info.put("isEnvironmentLookup", true);
            } else {
//                queue.add(info);
            }

        }

    }

    /**
     * This method takes a implication and checks whether the rule is an
     * environment lookup or not.
     *
     * The algorithm assumes that a rule is a lookup and tries to contradict it.
     *
     * @param imp
     * @return
     */
    private boolean isEnvironmentLookup(KIFImplication imp) {
        boolean isLookup = true;

        Stack<ASTNode> stack = new Stack<ASTNode>();
        for (KIFSent premise : imp.getPremise() )
            stack.push(premise);


        while ( !stack.isEmpty() ) {

            ASTNode node = stack.pop();

            switch ( node.getNodeType() ) {
            case ASTNode.CONJUNCTION:
              for (KIFSent operand : ((KIFConjunction) node).getOperands() )
                  stack.push(operand);
                  break;
            case ASTNode.DISJUNCTION:
                for (KIFSent operand : ((KIFDisjunction) node).getOperands() )
                    stack.push(operand);
                break;
            case ASTNode.NEGATION:
                stack.push( ((KIFNegation) node).getOperand() );
                break;
            case ASTNode.FUNTERM:
                break;
            case ASTNode.RELSENT:
                KIFRelSent sent = (KIFRelSent) node;
                if ( !"true".equalsIgnoreCase(sent.getName()) ) {

                    // retrieve info for predicate
                    KBEntry info;
                    info = kb.get(sent.getName());

                    if ( info.get("isEnvironmentLookup") == null )
                        return false;

                }
                break;
            }

        }
        return isLookup;
    }

}
