package org.eclipse.palamedes.gdl.core.model;


/**
 * The IGameTree interface provides a facade to work with a game state space 
 * representation.
 */
public interface IGameTree {

    /**
     * Adds a new state to the tree and returns the new generated node. The
     * moves that lead to it are attached to the new node as well.
     */
    public IGameNode addChild(IGameNode parent, IGameState child, IMove[] moves);

    /**
     * Returns the child game node of a parent which was derived through moves.
     * If no such node exists the method should return null.
     */
    public IGameNode getNextNode(IGameNode parent, IMove[] moves);

    /**
     * Returns the root node of the game tree. For each game tree a root node
     * should exist.
     */
    public IGameNode getRootNode();

    /**
     * Announces a terminal state.
     */
    public void setTerminal(IGameState terminalState);


}
