package org.eclipse.palamedes.gdl.core.model;


/**
 * Classes that implement the IReasoner interface are responsible for retrieving 
 * the legal moves and the next states. If you want to use an own reasoner this
 * is the Interface you have to code against.
 */
public interface IReasoner {

    /**
     * Returns the legal moves for a specified role.
     * @throws InterruptedException
     */
    public IMove[] getLegalMoves(String roleName, IGameState currentState)
    throws InterruptedException;

    /**
     * Returns the next state for a given parent state and specified moves. 
     * @throws InterruptedException
     */
    public IGameState getNextState(IGameState currentState, IMove[] moves)
    throws InterruptedException;

    /**
     * Returns a value between 0 - 100 if a goal value exists; otherwise -1.
     */
    public int getGoalValue(String roleName, IGameState nextState)
    throws InterruptedException;

    /**
     * This method should return a valid FluentAdapter for this reasoner.
     */
    public IFluent createFluent(String fluentStr);

    /**
     * This method should return a valid MoveAdapter for this reasoner.
     * 
     * String should contain "(does <player> <move>)"
     */
    public IMove createMove(String moveStr);

}
