package org.eclipse.palamedes.gdl.core.resolver.javaprover;

import org.eclipse.palamedes.gdl.core.model.IFluent;
import org.eclipse.palamedes.gdl.core.model.IGameState;
import org.eclipse.palamedes.gdl.core.model.IMove;
import org.eclipse.palamedes.gdl.core.model.IReasoner;
import org.eclipse.palamedes.gdl.core.model.utils.GenericMove;

import cs227b.teamIago.gameProver.GameSimulator;
import cs227b.teamIago.parser.Parser;
import cs227b.teamIago.resolver.Atom;
import cs227b.teamIago.resolver.ExpList;
import cs227b.teamIago.resolver.Expression;
import cs227b.teamIago.resolver.Predicate;
import cs227b.teamIago.resolver.Theory;
import cs227b.teamIago.resolver.Variable;
import cs227b.teamIago.util.GameState;


public class ReasonerAdapter implements IReasoner {

    private static final class MoveAdapter extends GenericMove<Expression> {
        public MoveAdapter(Expression exp) {
            super(new TermAdapter(exp));
        }
    }

    GameSimulator simulator = new GameSimulator(false, false);


    public ReasonerAdapter(String gameDescription) {
        simulator.ParseDescIntoTheory(gameDescription);
    }

    public int getGoalValue(String roleName, IGameState currentState)
    throws InterruptedException {
    	// we can not simply use simulator.getGoalValue(...) because it returns 0 instead of -1 if no goal value was found  
        simulator.SetGameState( (GameState) currentState.getNativeState() );
        Theory theory=simulator.getTheory();
        ExpList goalArgs = new ExpList();
        goalArgs.add(new Atom(roleName));
        Variable vX=new Variable("X");
        goalArgs.add(vX);
		Expression e;
		try {
			e = theory.findx(vX,new Predicate(new Atom("goal"),goalArgs));
		} catch (InterruptedException ie) {
			e  = null;
		}
		if (e == null) return -1;
		else return Integer.parseInt(e.toString());
    }

    public IMove[] getLegalMoves(String roleName, IGameState currentState)
    throws InterruptedException {
        simulator.SetGameState( (GameState) currentState.getNativeState() );
        ExpList legalMoves = simulator.GetLegalMoves( new Atom(roleName) );

        if (legalMoves == null) {
            System.out.println("Warning: null result from Reasoner!");
            return new IMove[0];
        }

        IMove[] moves      = new IMove[legalMoves.size()];
        for (int i = 0; i < legalMoves.size(); i++)
            moves[i] = new MoveAdapter( (Predicate) legalMoves.get(i));

        return moves;
    }

    public IGameState getNextState(IGameState currentState, IMove[] moves)
    throws InterruptedException {

        // create native move list
        ExpList moveList = new ExpList();
        for (IMove move : moves)
            moveList.add((((MoveAdapter)move).getNativeMove()));

        // generate next state
        simulator.SetGameState((GameState) currentState.getNativeState());
        simulator.SimulateStep(moveList);
        StateAdapter state = new StateAdapter(simulator.GetGameState());

        if (simulator.IsTerminal())
            state.setTerminal();

        return state;
    }

    public IFluent createFluent(String fluentStr) {
        return new FluentAdapter((Predicate)Parser.parseDesc(fluentStr).get(0));
    }

    public IMove createMove(String moveStr) {
        return new MoveAdapter((Predicate)Parser.parseDesc(moveStr).get(0));
    }

}
