/*
    Copyright (C) 2008 Stephan Schiffel <stephan.schiffel@gmx.de>

    This file is part of Palamedes-IDE.

    Palamedes-IDE is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Palamedes-IDE is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Palamedes-IDE.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.eclipse.palamedes.gdl.core.resolver.javaprover;

import java.util.List;

import org.eclipse.palamedes.gdl.core.model.utils.TermWrapper;
import org.eclipse.palamedes.kif.core.ast.AST;
import org.eclipse.palamedes.kif.core.ast.KIFSeq;
import org.eclipse.palamedes.kif.core.ast.KIFTerm;

import cs227b.teamIago.resolver.Atom;
import cs227b.teamIago.resolver.Connective;
import cs227b.teamIago.resolver.Expression;
import cs227b.teamIago.resolver.Variable;

public class TermAdapter extends TermWrapper<Expression>{
	
	public TermAdapter(Expression nativeTerm){
		super(nativeTerm);
	}

	@Override
	public KIFTerm getKIFTerm() {
		return getKIFTerm(nativeTerm);
	}

	@SuppressWarnings("unchecked")
	public static final KIFTerm getKIFTerm(Expression expr) {
		if(expr instanceof Variable){
			return new AST().newIndVar("v"+Long.toString(((Variable)expr).getNum()));
		}else if(expr instanceof Atom){
			return new AST().newObjConst(expr.toString().toLowerCase());
		}else if(expr instanceof Connective){
			Connective c=(Connective)expr;
			KIFSeq<KIFTerm> args=new KIFSeq<KIFTerm>();
			for(Expression e:(List<Expression>)c.getOperands().toArrayList()){
				args.add(getKIFTerm(e));
			}
			return new AST().newFunTerm(c.getOperator().toString().toLowerCase(), args);
		}
		return null;
	}

}
