package org.eclipse.palamedes.gdl.core.resolver.prologprover;

import java.io.File;
import java.io.IOException;
import java.util.Collection;

import com.parctechnologies.eclipse.CompoundTerm;
import com.parctechnologies.eclipse.CompoundTermImpl;
import com.parctechnologies.eclipse.EclipseEngineOptions;
import com.parctechnologies.eclipse.EclipseException;
import com.parctechnologies.eclipse.EmbeddedEclipse;
import com.parctechnologies.eclipse.Fail;

public class EclipseConnector {
	// Create some default Eclipse options
	private EclipseEngineOptions eclipseEngineOptions;

	// Object representing the Eclipse process
	protected EmbeddedEclipse eclipse;

	private String
		ECLIPSE_SOURCE_DIR="prologsource/prolog_reasoner.ecl",
		ECLIPSE_DIR;

	private EclipseConnector() {
		ECLIPSE_DIR=System.getenv("ECLIPSE_DIR");
		if(ECLIPSE_DIR==null){
			ECLIPSE_DIR="/usr/local/eclipse";
		}
		initEclipse();
	}
	
	private static EclipseConnector instance=null;
	public static EclipseConnector getInstance(){
		if(instance==null){
			instance=new EclipseConnector();
		}
		return instance;
	}

	public void initEclipse() {

		System.setProperty("eclipse.directory", ECLIPSE_DIR);

		eclipseEngineOptions = new EclipseEngineOptions();

		eclipseEngineOptions.setUseQueues(false);
		eclipseEngineOptions.setGlobalSize(64);
		eclipseEngineOptions.setLocalSize(64);

		// Initialise Eclipse
		try {
			eclipse = EmbeddedEclipse.getInstance(eclipseEngineOptions);
			System.out.println("Compiling \""+ECLIPSE_SOURCE_DIR+"\"");
			eclipse.compile(new File(ECLIPSE_SOURCE_DIR));
			eclipse.rpc("use_module(prolog_reasoner)");

		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("initEclipse done.");
	}
	
	public CompoundTerm parseTerm(String s) {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc(new CompoundTermImpl("parse",s,null));
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// TODO Auto-generated method stub
		return (CompoundTerm)result.arg(2);
	}

	public String toGdlString(Object prologTerm) {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc(new CompoundTermImpl("translate_to_sexpr",prologTerm,null));
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return (String)result.arg(2);
	}

	public int getGoalValue(CompoundTerm role, Collection<CompoundTerm> state) {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc(new CompoundTermImpl("goal",role,null,state));
		} catch (Fail e){
			return -1;
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return ((Integer)result.arg(2)).intValue();
	}

	@SuppressWarnings("unchecked")
	public Collection<CompoundTerm> getLegalMoves(CompoundTerm role, Collection<CompoundTerm> state) {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc(new CompoundTermImpl("legal_moves",role,null,state));
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return (Collection<CompoundTerm>)result.arg(2);
	}

	@SuppressWarnings("unchecked")
	public Collection<CompoundTerm> getNextState(Collection<CompoundTerm> movesList, Collection<CompoundTerm> state) {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc(new CompoundTermImpl("state_update",state,movesList,null));
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return (Collection<CompoundTerm>)result.arg(3);
	}

	public boolean isTerminal(Collection<CompoundTerm> state) {
		try {
			eclipse.rpc(new CompoundTermImpl("terminal", state));
		} catch (Fail e) {
			return false;
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return true;
	}

	public void parseGDL(String gameDescription) {
		try {
			eclipse.rpc(new CompoundTermImpl("setgame",gameDescription));
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	public Collection<CompoundTerm> getInitialState() {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc("init(State)");
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return (Collection<CompoundTerm>)result.arg(1);
	}

	@SuppressWarnings("unchecked")
	public Collection<CompoundTerm> getRoles() {
		CompoundTerm result=null;
		try {
			result = eclipse.rpc("roles(Roles)");
		} catch (EclipseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return (Collection<CompoundTerm>)result.arg(1);
	}

}
