package org.eclipse.palamedes.gdl.core.resolver.prologprover;

import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.palamedes.gdl.core.model.IFluent;
import org.eclipse.palamedes.gdl.core.model.IGameState;
import org.eclipse.palamedes.gdl.core.model.IMove;
import org.eclipse.palamedes.gdl.core.model.IReasoner;

import com.parctechnologies.eclipse.Atom;
import com.parctechnologies.eclipse.CompoundTerm;

public final class ReasonerAdapter implements IReasoner {

    private final EclipseConnector eclipse;

    public ReasonerAdapter(String gameDescription) {
    	eclipse=EclipseConnector.getInstance();
        eclipse.parseGDL(gameDescription);
    }

	public IFluent createFluent(String fluentStr) {
		return new FluentAdapter(eclipse.parseTerm(fluentStr));
	}

	public IMove createMove(String moveStr) {
		return new MoveAdapter(eclipse.parseTerm(moveStr));
	}

	public int getGoalValue(String roleName, IGameState gameState)
		throws InterruptedException {
		StateAdapter state=(StateAdapter)gameState;
		return eclipse.getGoalValue(new Atom(roleName), state.getNativeState());
	}

	public IMove[] getLegalMoves(String roleName, IGameState gameState)
			throws InterruptedException {
		StateAdapter state=(StateAdapter)gameState;
		Collection<CompoundTerm> movesList=eclipse.getLegalMoves(new Atom(roleName), state.getNativeState());
		IMove[] moves=new IMove[movesList.size()];
		int i=0;
		for(CompoundTerm prologMove : movesList){
			moves[i]=new MoveAdapter(prologMove);
			i++;
		}
		return moves;
	}

	public IGameState getNextState(IGameState currentState, IMove[] moves)
			throws InterruptedException {
		StateAdapter state=(StateAdapter)currentState;
		ArrayList<CompoundTerm> movesList=new ArrayList<CompoundTerm>(moves.length);
		for(IMove move:moves){
			movesList.add(((MoveAdapter)move).getNativeMove());
		}
		StateAdapter nextState=new StateAdapter(eclipse, eclipse.getNextState(movesList, state.getNativeState()));
		if(eclipse.isTerminal(nextState.getNativeState())){
			nextState.setTerminal();
		}
		return nextState;
	}

	public IGameState getInitialState() {
		StateAdapter nextState=new StateAdapter(eclipse, eclipse.getInitialState());
		if(eclipse.isTerminal(nextState.getNativeState())){
			nextState.setTerminal();
		}
		return nextState;
	}

	public String[] getRoleNames() {
		Collection<CompoundTerm> rolesList=eclipse.getRoles();
		String[] roles=new String[rolesList.size()];
		int i=0;
		for(CompoundTerm role : rolesList){
			roles[i]=role.functor();
			i++;
		}
		return roles;
	}


}
