package org.eclipse.palamedes.gdl.core.simulation;

import org.eclipse.palamedes.gdl.core.model.IGameNode;
import org.eclipse.palamedes.gdl.core.model.IMove;


public interface IStrategy {

    public void initMatch(Match match);

    public IMove getMove(IGameNode currentNode);

    /**
     * Returns a value for the game node that can be used by a heuristic to
     * evaluate the game node.
     *
     * If no such value can be generated then -1 is the default return value.
     */
    public double getHeuristicValue(IGameNode node);

    
    /**
     * A strategy is responsible to dispose all own references to avoid 
     * reference cycles.
     */
    public void dispose();

    
    /**
     * Returns a value for the last chosen move that should give an indicator
     * value for the reliability of it.
     */
    public double getReliabilityValue();

    /**
     * Returns a value for the last chosen move that should give an indicator
     * what goal value we think we might get in the end.
     */
    public double getExpectedGoalValue();

}
