package org.eclipse.palamedes.gdl.core.simulation;

import java.util.ArrayList;
import java.util.List;
import java.util.Timer;

import org.eclipse.palamedes.gdl.core.model.IGame;
import org.eclipse.palamedes.gdl.core.model.IGameNode;
import org.eclipse.palamedes.gdl.core.model.IMove;
import org.eclipse.palamedes.gdl.core.model.utils.GameNodeTree;


public class Match {

    private final IGame     game;
    private final IStrategy strategy;
    private final String    role;
    private final int       startTime;
    private final int       playTime;
    private final TimerFlag timer;

    private final List<IGameNode> history = new ArrayList<IGameNode>();

    IGameNode currentNode;

    public Match(IGame game, IStrategy strategy, String role, int tStart, int tPlay, IGameNode startNode) {
        super();

        Timer t = null;

        // to be precise as possible the first thing we do is to start the timer
        timer = new TimerFlag();
        if (tStart > -1) {
            t = new Timer();
            t.schedule( new TimerThread(timer), tStart * 1000L);
        }

        this.game        = game;
        this.strategy    = strategy;
        this.role        = role;
        this.startTime   = tStart;
        this.playTime    = tPlay;
        this.currentNode = startNode;

        this.strategy.initMatch(this);

        // cancel timer
        if (t != null) {
        	t.cancel();
        }
    }
    
    public Match(IGame game, IStrategy strategy, String role, int tStart, int tPlay) {
        this(game, strategy, role, tStart, tPlay, game.getTree().getRootNode());
    }

    public void makeTurn(IMove[] moves) {
        try {
            currentNode = game.getNextNode(currentNode, moves);
            currentNode.setPreserve(true);
            history.add(currentNode);
            GameNodeTree tree = (GameNodeTree) game.getTree();
            tree.removeAllSiblingTrees(currentNode);
        } catch (InterruptedException exp) {
            exp.printStackTrace();
        }
    }


    public void makeTurn(String[] moveStr) {
        IMove[] moves = new IMove[moveStr.length];
        for (int i = 0; i < moveStr.length; i++)
            moves[i] = game.getReasoner().createMove(moveStr[i]);

        makeTurn(moves);
    }

    public double getReliabilityValue() {
        return strategy.getReliabilityValue();
    }

    public IMove getMove() {
    	Timer t = null;
        // start timer
        if (playTime > -1) {
            t=new Timer();
        	t.schedule( new TimerThread(timer), playTime * 1000);
        }

        IMove move = strategy.getMove( getCurrentNode() );

        // cancel timer
        if (t != null) {
        	t.cancel();
        }
        return move;
    }


    public IGame getGame() {
        return game;
    }


    public IStrategy getStrategy() {
        return strategy;
    }


    public String getRole() {
        return role;
    }


    public int getStartTime() {
        return startTime;
    }


    public int getPlayTime() {
        return playTime;
    }


    public List<IGameNode> getHistory() {
        return history;
    }


    public IGameNode getCurrentNode() {
        return currentNode;
    }

    public void setCurrentNode(IGameNode newNode) {
        currentNode = newNode;
    }

    public TimerFlag getTimer() {
        return timer;
    }

    public int getGoalValue() {
        if ( !currentNode.isTerminal() )
            return -1;
        
//        // regenerate if necessary
//        if (currentNode.getState() == null) {
//            
//        }
        return currentNode.getState().getGoalValue(game.getRoleIndex(role));
    }

}
