package org.eclipse.palamedes.gdl.core.simulation;

import org.eclipse.palamedes.gdl.core.model.utils.AbstractFactory;


/**
 * The Strategy Factory is responsible for creating strategy objects on demand.
 * The factory is implemented as singleton.
 *
 * A implementation of IStrategy can be registered at the StrategyFactory.
 *
 */
public class StrategyFactory extends AbstractFactory {

	private static class InstanceHolder {
		public static StrategyFactory instance = new StrategyFactory();
	}

    /** This field is just for convenience **/
    private static final String defaultPackage =
                        "org.eclipse.palamedes.gdl.core.simulation.strategies";

    /**
     * Private constructor to avoid instantiation outside this class.
     */
    private StrategyFactory() {
     
    	// register default implementations strategies
        addDescription( "Random",
                        defaultPackage + ".SRandom",
                        "Plain Random Move Choice" );

        addDescription( "DeepFirstSearch",
                        defaultPackage + ".SDFS",
                        "Tries to reach the best goal." );

        addDescription( "Monte Carlo",
                        defaultPackage + ".SMonteCarlo",
                        "Simulates games and chooses the best path." );

        addDescription( "Monte Carlo UCT",
                        defaultPackage + ".SMonteCarloUCT",
                        "Simulates games and chooses the best path." );
    }

    /**
     * Returns the instance of the strategy factory.
     */
    public static StrategyFactory getInstance() {
        return InstanceHolder.instance;
    }

    public IStrategy createStrategy(int index) {
        return createStrategy(descriptions.get(index));
    }

    public IStrategy createStrategy(String name) {
        return createStrategy(getDescriptionByName(name));
    }

    public IStrategy createStrategy(FactoryDescription description) {
    	return (IStrategy) createInstance(description);
    }

}
