package org.eclipse.palamedes.gdl.core.simulation.strategies;

import java.util.List;
import java.util.Random;

import org.eclipse.palamedes.gdl.core.model.IGame;
import org.eclipse.palamedes.gdl.core.model.IGameNode;
import org.eclipse.palamedes.gdl.core.model.IMove;
import org.eclipse.palamedes.gdl.core.simulation.IStrategy;
import org.eclipse.palamedes.gdl.core.simulation.Match;


public abstract class AbstractStrategy implements IStrategy {

    /** A random number generator. */
    protected static final Random random = new Random();

    /** The upper limit of expected steps for a game */
    protected static final int maxStepsDefault = 500;

    /** General interrupted exception for timing purpose */
    protected static final InterruptedException iException = 
    												new InterruptedException();

    protected Match  match;
    protected IGame  game;
    protected int    maxSteps;
    protected int    playerNumber;
    protected double reliability;
    protected double heuristic;
    protected double expectedGoal;

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.gdl.core.simulation.IStrategy#initMatch(org.eclipse.palamedes.gdl.core.simulation.Match)
     */
    public void initMatch(Match initMatch) {
        match        = initMatch;
        game         = match.getGame();
        heuristic    = -1;
        reliability  = -1;
        expectedGoal = 0;

        // how many steps has the game at maximum ?
        maxSteps = match.getGame().getMaxSteps();
        maxSteps = maxSteps == -1 ? maxStepsDefault : maxSteps;

        // which player number do we have ?
        playerNumber = match.getGame().getRoleIndex(match.getRole());
        if(playerNumber==-1){
        	System.err.println("Can't compute player number!");
        	Thread.dumpStack();
        }
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.gdl.core.simulation.IStrategy#getMove(org.eclipse.palamedes.gdl.core.model.IGameNode)
     */
    public abstract IMove getMove (IGameNode currentNode);

    /*
     * (non-Javadoc)
     * @see org.eclipse.palamedes.gdl.core.simulation.IStrategy#dispose()
     */
    public void dispose() {
        match = null;
        game  = null;
    }

    public double getHeuristicValue(IGameNode node) {
        return heuristic;
    }
    
    public double getReliabilityValue() {
        return reliability;
    }
    
    public double getExpectedGoalValue() {
        return expectedGoal;
    }

    public IMove[] getRandomCombinedMove()
    throws InterruptedException {
        List<IMove[]> moveList = game.getCombinedMoves(match.getCurrentNode());
        return moveList.get(random.nextInt(moveList.size()));
    }

}
