:- module(prolog_reasoner).

:- export
	parse/2,
	translate_to_sexpr/2,
	goal/3,
	legal_moves/3,
	state_update/3,
	terminal/1,
	setgame/1,
	init/1,
	roles/1.

:- use_module(gdl_parser).
:- use_module(game_description_interface).

:- lib(lists).

parse(String, Term) :-
	parse_gdl_term_string(String, Term).

translate_to_sexpr(Term, String) :-
	translate_to_sexpr_string(Term, String).

goal(Role, Value, State) :-
	d_goal(Role, Value, nil-State).

legal_moves(Role, Moves, State) :-
	fast_setof(does(Role, M), d_legal(Role, M, nil-State), Moves).

state_update(State, Moves, NextState) :-
	fast_setof(F, d_next(F, Moves-State), NextState).

terminal(State) :-
	d_terminal(nil-State).
	
setgame(GDLDescriptionString) :-
	parse_gdl_description_string(GDLDescriptionString, Rules),
	compile_new_rules(Rules).

init(State) :-
	fast_setof(F, d_init(F, nil-nil), State).
	
roles(Roles) :-
	findall(R, d_role(R, nil-nil), Roles).
	
fast_setof(X,Expr,Xs) :-
	findall(X,Expr,Xs1),
	sort(0,<,Xs1,Xs).
	

